package Channel;

use strict;
use Carp;
use warnings;

require ProviderInfo;
require Channel;

my $Debugging = 0;

my $ChannelCount = 0;
my @ChannelList;

my $IconPrefix;

# the object constructor

sub new {
	my $class = shift;
	my $self  = {};
	my @empty;
	$self->{NAME}   = "";
	$self->{LONGNAME}    = "";
	$self->{ALTNAME}    = "foo";
	$self->{UKRTID}    = "bar";
	$self->{ICON}    = "";
	$self->{ICONURL}    = "baz";
	$self->{XMLTVID}    = "";
	$self->{PROVIDERINFO}    = [];
	# "private" data
	$self->{"_ICONPREFIX"} = \$IconPrefix;
	$self->{"_CHANNELCOUNT"} = \$ChannelCount;
	$self->{"_CHANNELLIST"}  = \@ChannelList;
	bless ($self, $class);
	push @{ $self->{"_CHANNELLIST"} }, $self;
	++ ${ $self->{"_CHANNELCOUNT"} };
	return $self;
}

sub iconprefix {
	my $self = shift;
	if (@_) {
		if (ref $self) {
			${ $self->{"_ICONPREFIX"} } = shift
		} else {
			$IconPrefix = shift;
		}
	} else {
		if (ref $self) {
			return ${ $self->{"_ICONPREFIX"} };
		} else {
			return $IconPrefix;
		}
	}
}

sub findByName {
	my $self = shift;
	confess "usage: thing->findByName(search)"    unless @_ == 1;
	my $search = shift;
	for my $o (@ChannelList) {
		if ($o->name() eq $search) {
			return $o;
		}
	}
}

sub findByLongname {
	my $self = shift;
	confess "usage: thing->findByLongname(search)"    unless @_ == 1;
	my $search = shift;
	for my $o (@ChannelList) {
		if ($o->longname() eq $search) {
			return $o;
		}
	}
}

sub findByAltname {
	my $self = shift;
	confess "usage: thing->findByAltname(search)"    unless @_ == 1;
	my $search = shift;
	for my $o (@ChannelList) {
		if ($o->altname() eq $search) {
			return $o;
		}
	}
}

sub findByXMLTVID {
	my $self = shift;
	confess "usage: thing->findByXMLTVID(search)"    unless @_ == 1;
	my $search = shift;
	for my $o (@ChannelList) {
		if ($o->xmltvid() eq $search) {
			return $o;
		}
	}
}

# methods to access per-object data
#
# With args, they set the value.  Without
# any, they only retrieve it/them.

sub name {
	my $self = shift;
	if (@_) { $self->{NAME} = shift }
	return $self->{NAME};
}

sub longname {
	my $self = shift;
	if (@_) { $self->{LONGNAME} = shift }
	return $self->{LONGNAME};
}

sub altname {
	my $self = shift;
	if (@_) { $self->{ALTNAME} = shift }
	return $self->{ALTNAME};
}

sub ukrtid {
	my $self = shift;
	if (@_) { $self->{UKRTID} = shift }
	return $self->{UKRTID};
}

sub icon {
	my $self = shift;
	if (@_) { $self->{ICON} = shift }
	return $self->{ICON};
}

sub iconurl {
	my $self = shift;
	if (@_) { $self->{ICONURL} = shift }
	return $self->{ICONURL};
}

sub xmltvid {
	my $self = shift;
	if (@_) { $self->{XMLTVID} = shift }
	return $self->{XMLTVID};
}

sub providerinfo {
	my $self = shift;
	if (@_) { push @{$self->{PROVIDERINFO}}, shift }
	return $self->{PROVIDERINFO};
}

sub listOfChannels {
	my $self = shift;
	if (ref $self) {
		return @{ $self->{"_CHANNELLIST"} };
	} else {
		return @ChannelList;
	}
}

sub noOfChannels {
	my $self = shift;
	if (ref $self) {
		return ${ $self->{"_CHANNELCOUNT"} };
	} else {
		return $ChannelCount;
	}
}

sub toString {
	my $self = shift;
	my $s = "";
	if (ref $self) {
		my $t = "";
		foreach my $pi (@{$self->{PROVIDERINFO}}) {
			$t = $t . $pi->toString();
		}
		$s =
			"{" .
			" name=\"" . $self->{NAME} . "\"" .
			" longname=\"" . $self->{LONGNAME} . "\"" .
			" altname=\"" . $self->{ALTNAME} . "\"" .
			" ukrtid=\"" . $self->{UKRTID} . "\"" .
			" icon=\"" . $self->{ICON} . "\"" .
			" iconurl=\"" . $self->{ICONURL} . "\"" .
			" xmltvid=\"" . $self->{XMLTVID} . "\"" .
			" providerinfo=\"" . $t . "\"" .
			" }";
	} else {
		for my $o (@ChannelList) {
			$s = $s . "\n" . $o->toString();
		}
	}
	return $s;
}

sub debug {
	my $self = shift;
	confess "usage: thing->debug(level)"    unless @_ == 1;
	my $level = shift;
	if (ref($self))  {
		$self->{"_DEBUG"} = $level;         # just myself
	} else {
		$Debugging        = $level;         # whole class
	}
}

sub DESTROY {
	my $self = shift;
	if ($Debugging || $self->{"_DEBUG"}) {
		carp "Destroying $self " . $self->name;
	}
	-- ${ $self->{"_CHANNELCOUNT"} };
}

sub END {
	if ($Debugging) {
		print "All persons are going away now.\n";
	}
}

1;  # so the require or use succeeds
