package ChannelXML;

use strict;
use Carp;
use warnings;

use XML::LibXML;

require Channel;
require Provider;
require ProviderInfo;
require TunerFactory;

sub load {
	my $self = shift;
	confess "usage: ChannelXML->load(node)"    unless @_ == 1;
	my $node = shift;
	my @channels = $node->getElementsByTagName('channel');

	foreach my $channel (@channels) {
		my $ch = Channel->new();
		$ch->name($channel->findvalue('@name'));
		$ch->longname($channel->findvalue('longname'));
		$ch->altname($channel->findvalue('altname'));
		$ch->ukrtid($channel->findvalue('ukrtid'));
		$ch->icon($channel->findvalue('icon'));
		$ch->iconurl($channel->findvalue('iconurl'));
		$ch->xmltvid($channel->findvalue('xmltvid'));
		foreach my $provider ($channel->findnodes('providerinfo')) {
			my $pi = ProviderInfo->new();
			$pi->name($provider->findvalue('@name'));
			$pi->channelnumber($provider->findvalue('channelnumber'));
			my $ti = TunerFactory->getTunerInfo($pi->name());
			my $tx = TunerFactory->getTunerXML($pi->name());
			$tx->load($provider, $ti);
			$pi->tunerinfo($ti);
			$ch->providerinfo($pi);
		}
	}
}

sub store {
	my $self = shift;
	confess "usage: ChannelXML->store(fd)"    unless @_ == 1;
	my $fd = shift;

	my @cl = Channel->listOfChannels();
	foreach my $ch (@cl) {
		my $ch_name = $ch->name();
		my $ch_longname = $ch->longname();
		my $ch_altname = $ch->altname();
		my $ch_ukrtid = $ch->ukrtid();
		my $ch_icon = $ch->icon();
		my $ch_iconurl = $ch->iconurl();
		my $ch_xmltvid = $ch->xmltvid();
		print $fd "\t<channel name=\"" . $ch_name . "\">\n";
		print $fd "\t\t<longname>" . $ch_longname . "</longname>\n";
		print $fd "\t\t<altname>" . $ch_altname . "</altname>\n";
		print $fd "\t\t<ukrtid>" . $ch_ukrtid . "</ukrtid>\n";
		print $fd "\t\t<icon>" . $ch_icon . "</icon>\n";
		print $fd "\t\t<iconurl>" . $ch_iconurl . "</iconurl>\n";
		print $fd "\t\t<xmltvid>" . $ch_xmltvid . "</xmltvid>\n";
		my @providerinfo = @{$ch->providerinfo()};
		foreach my $pi (@providerinfo) {
			my $pi_channelnumber = $pi->channelnumber();
			my $pi_name = $pi->name();
			my $pr = Provider->findByName($pi_name);
			my $pr_tuner = $pr->tuner();
			my $pr_chanid = $pr->chanid();
			print $fd "\t\t<providerinfo name=\"" . $pi_name . "\">\n";
			print $fd "\t\t\t<channelnumber>" . $pi_channelnumber . "</channelnumber>\n";
			print $fd "\t\t\t<tunerinfo>\n";
			if ($pr_tuner eq 'external') {
				my $external_epgnumber = $pi->tunerinfo()->epgnumber();
				print $fd "\t\t\t\t<epgnumber>" . $external_epgnumber . "</epgnumber>\n";
			} elsif ($pr_tuner eq 'analogue') {
				my $analogue_frequency = $pi->tunerinfo()->frequency();
				print $fd "\t\t\t\t<frequency>" . $analogue_frequency . "</frequency>\n";
			} elsif ($pr_tuner eq 'dvb_s') {
				my $dvb_s_mplexid = $pi->tunerinfo()->mplexid();
				my $dvb_s_serviceid = $pi->tunerinfo()->serviceid();
				my $dvb_s_useonairguide = $pi->tunerinfo()->useonairguide();
				print $fd "\t\t\t\t<mplexid>" . $dvb_s_mplexid . "</mplexid>\n";
				print $fd "\t\t\t\t<serviceid>" . $dvb_s_serviceid . "</serviceid>\n";
				print $fd "\t\t\t\t<useonairguide>" . $dvb_s_useonairguide . "</useonairguide>\n";
			}
			print $fd "\t\t\t</tunerinfo>\n";
			print $fd "\t\t</providerinfo>\n";
		}
		print $fd "\t</channel>\n";
	}
}

1;
