package Provider;

use strict;
use Carp;
use warnings;

my $Debugging = 0;

my $ProviderCount = 0;
my @ProviderList;

# the object constructor

sub new {
	my $class = shift;
	my $self  = {};
	$self->{NAME}   = "";
	$self->{TYPE}    = "";
	$self->{TUNER}    = "";
	$self->{CHANID}    = "";
	$self->{DESCRIPTION}    = "";
	# "private" data
	$self->{"_PROVIDERCOUNT"} = \$ProviderCount;
	$self->{"_PROVIDERLIST"}  = \@ProviderList;
	bless ($self, $class);
	push @{ $self->{"_PROVIDERLIST"} }, $self;
	++ ${ $self->{"_PROVIDERCOUNT"} };
	return $self;
}

# methods to access per-object data
#
# With args, they set the value.  Without
# any, they only retrieve it/them.

sub name {
	my $self = shift;
	if (@_) { $self->{NAME} = shift }
	return $self->{NAME};
}

sub type {
	my $self = shift;
	if (@_) { $self->{TYPE} = shift }
	return $self->{TYPE};
}

sub tuner {
	my $self = shift;
	if (@_) { $self->{TUNER} = shift }
	return $self->{TUNER};
}

sub chanid {
	my $self = shift;
	if (@_) { $self->{CHANID} = shift }
	return $self->{CHANID};
}

sub description {
	my $self = shift;
	if (@_) { $self->{DESCRIPTION} = shift }
	return $self->{DESCRIPTION};
}

sub listOfProviders {
	my $self = shift;
	if (ref $self) {
		return @{ $self->{"_PROVIDERLIST"} };
	} else {
		return @ProviderList;
	}
}

sub noOfProviders {
	my $self = shift;
	if (ref $self) {
		return ${ $self->{"_PROVIDERCOUNT"} };
	} else {
		return $ProviderCount;
	}
}

sub findByName {
	my $self = shift;
	confess "usage: thing->findByName(search)"    unless @_ == 1;
	my $search = shift;
	for my $o (@ProviderList) {
		if ($o->name() eq $search) {
			return $o;
		}
	}
}

sub toString {
	my $self = shift;
	my $s = "";
	if (ref $self) {
		$s =
			"{" .
			" name=\"" . $self->{NAME} . "\"" .
			" type=\"" . $self->{TYPE} . "\"" .
			" tuner=\"" . $self->{TUNER} . "\"" .
			" chanid=\"" . $self->{CHANID} . "\"" .
			" description=\"" . $self->{DESCRIPTION} . "\"" .
			" }";
	} else {
		for my $o (@ProviderList) {
			$s = $s . "\n" . $o->toString();
		}
	}
	return $s;
}

sub debug {
	my $self = shift;
	confess "usage: thing->debug(level)"    unless @_ == 1;
	my $level = shift;
	if (ref($self))  {
		$self->{"_DEBUG"} = $level;         # just myself
	} else {
		$Debugging        = $level;         # whole class
	}
}

sub DESTROY {
	my $self = shift;
	if ($Debugging || $self->{"_DEBUG"}) {
		carp "Destroying $self " . $self->name;
	}
	-- ${ $self->{"_PROVIDERCOUNT"} };
}

sub END {
	if ($Debugging) {
		print "All persons are going away now.\n";
	}
}

1;  # so the require or use succeeds
